import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'base_view_state.dart';

mixin BaseMixRequestModelController on BaseViewState{

  List<RequestInfo> getRequestInfoList();

  List<HttpData> httpDataList(){
    List<HttpData> httpDataList = [];
    _httpDataCollection.forEach((key, value) {
      if(isEmpty(value.httpError)){
        httpDataList.add(value);
      }
    });
    return httpDataList;
  }

  Map<String, HttpData> _httpDataCollection = {};

  void getData(){

    List<RequestInfo> requestInfoList = getRequestInfoList();

    if(isNotEmpty(requestInfoList)){
      requestInfoList.forEach((requestInfo) async{
        if(isSync){
          ///同步请求
          HttpData httpData = await doRequest(requestInfo);
          _httpDataCollection[requestInfo.url] = httpData;
          onCompleted(httpDataList());
        }else{
          ///异步请求
          doRequest(requestInfo).then((httpData){
            _httpDataCollection[requestInfo.url] = httpData;
            onCompleted(httpDataList());
            setIdle();
            if(_httpDataCollection.length == requestInfoList.length){
              bool allError = true;
              List<HttpErrorType> httpErrorTypeList = [];
              _httpDataCollection.forEach((key, value){
                if(isEmpty(value.httpError)){
                  allError = false;
                }else{
                  httpErrorTypeList.add(value.httpError.httpErrorType);
                }
              });
              if(allError){
                HttpError httpError = HttpError(HttpErrorType.Unknown, HTTP_ERROR_MSG, logMsg: '异步多个请求失败： $httpErrorTypeList');
                setError(httpError);
              }
            }
          });
        }
      });
    }

    if(isSync){
      bool allError = true;
      List<HttpErrorType> httpErrorTypeList = [];
      _httpDataCollection.forEach((key, value){
        if(isEmpty(value.httpError)){
          allError = false;
        }else{
          httpErrorTypeList.add(value.httpError.httpErrorType);
        }
      });
      if(allError){
        HttpError httpError = HttpError(HttpErrorType.Unknown, HTTP_ERROR_MSG, logMsg: '异步多个请求失败： $httpErrorTypeList');
        setError(httpError);
      }else{
        setIdle();
      }
    }
  }




  onCompleted(List<HttpData> httpDataList) {}


  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      getData();
    }
  }


  ///默认异步请求
  bool get isSync => false;

  @override
  RequestInfo getRequestInfo() {
    return null;
  }


}