import 'package:dio/dio.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/utils/utils.dart';


class HttpError implements Exception {

  HttpErrorType httpErrorType;

  ///打印给开发者的msg
  String logMsg;

  ///可自定义，给用户的信息
  String message;

  Exception cause;

  HttpError(this.httpErrorType, this.message, {this.cause, this.logMsg});


  ///处理dio报错
  HttpError.dioError(DioError error) {
    this.cause = error;
    switch (error.type) {
      case DioErrorType.connectTimeout:
      case DioErrorType.receiveTimeout:
      case DioErrorType.sendTimeout:
        httpErrorType = HttpErrorType.TimeOutError;
        logMsg = '请求超时';
        message = HTTP_TIMEOUT_MSG;
        break;
      case DioErrorType.response:
        httpErrorType = HttpErrorType.ResponseError;
        message = HTTP_ERROR_MSG;
        try {
          int errCode = error.response.statusCode;
          switch (errCode) {
            case 400:
              logMsg = '400请求语法错误';
              break;
            case 401:
              logMsg = '401没有权限';
              break;
            case 403:
              logMsg = '403服务器拒绝执行';
              break;
            case 404:
              logMsg = '404无法连接服务器';
              break;
            case 405:
              logMsg = '405请求方法被禁止';
              break;
            case 500:
              logMsg = '500服务器内部错误';
              break;
            case 502:
              logMsg = '502无效的请求';
              break;
            case 503:
              logMsg = '503服务器挂了';
              break;
            case 505:
              logMsg = '505不支持HTTP协议请求';
              break;
            default:
              logMsg = error.response.statusMessage;
          }
        } on Exception catch (_) {
          logMsg = '未知错误';
        }
        break;
      case DioErrorType.cancel:
        httpErrorType = HttpErrorType.CancelError;
        logMsg = CANCEL_MSG;
        break;
      case DioErrorType.other:
        httpErrorType = HttpErrorType.Unknown;
        message = UNKNOWN_MSG + ': ' + error.error.toString();
        break;
    }
  }


  ///-------------------------------------------------------HttpError生成---------------------------------------------------
  HttpError.emptyDataError(data){
    httpErrorType = HttpErrorType.EmptyDataError;
    message = NO_DATA_MSG;
    logMsg = 'data数据为空: ${data.toString()}';
  }


  HttpError.parseDataError(e){
    httpErrorType = HttpErrorType.DataError;
    message = PARSE_ERROR_MSG;
    logMsg = '数据解析错误: $e';
    cause = e is Exception? e : null;
  }



  @override
  String toString() {
    return 'HttpError{type: $httpErrorType, errorMsg: $logMsg}';
  }
}


enum HttpErrorType {
  Unknown,
  DataError,
  EmptyDataError,
  NullBodyError,
  CancelError,
  TimeOutError,
  ResponseError,
  ResultError,
  TokenError,
  NetWorkError,
  SSLError
}


