import 'dart:convert';
import 'package:encrypt/encrypt.dart';
import 'package:flutter/foundation.dart' as platForm;
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:hive/hive.dart';
import 'package:path/path.dart';
import 'package:path_provider/path_provider.dart';
import 'package:pointycastle/export.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'keypair.dart';

enum StorageType{
  SP,
  HIVE
}


class StorageUtil {
  static StorageUtil _instance = StorageUtil._internal();
  SharedPreferences _prefs;
  Box _realEncryptBox;
  StorageType _storageType;
  Encrypter _realEncrypt;
  final _aesIv = IV.fromLength(16);
  bool _encrypt;
  factory StorageUtil() => _instance;


  // 私有构造函数
  StorageUtil._internal();


  Future init({bool encrypt = true, StorageType storageType = StorageType.HIVE, String hiveSubDir}) async{
    _storageType = storageType;
    _prefs = await SharedPreferences.getInstance();
    _encrypt = encrypt;
    if(encrypt){

      ///第一层加密
      final key = Key.fromUtf8(defPassword);
      final iv = IV.fromLength(8);
      final salsaEncrypt = Encrypter(Salsa20(key));
      String _firstKeyMD5;
      String _rsaKeyMD5 = getMD5('rsaKey');
      RSAPrivateKey _rsaPrivateKey;
      RSAPublicKey _rsaPublicKey;
      String _rsaPublicKeyStr;
      String _rsaPrivateKeyStr;
      Encrypter _rSAKeyEncrypt;


      if(_storageType == StorageType.HIVE){
        if (!platForm.kIsWeb) {
          var appDir = await getApplicationDocumentsDirectory();
          if (appDir != null) {
            Hive.init(join(appDir.path, hiveSubDir));
          } else {
            NullThrownError();
          }
        }

        ///需要加密
        List<int> _hiveKey;
        _firstKeyMD5 = getMD5('hiveKey');
        if(_prefs.containsKey(_firstKeyMD5)){
          String encryptBase64 = _prefs.getString(_firstKeyMD5);
          final decrypted = salsaEncrypt.decrypt64(encryptBase64, iv: iv);
          _hiveKey = base64.decode(decrypted);
        }else{
          _hiveKey = Hive.generateSecureKey();
          String hiveValueBase64 = salsaEncrypt.encrypt(base64.encode(_hiveKey), iv: iv).base64;
          _prefs.setString(_firstKeyMD5, hiveValueBase64);
        }

        ///保存rsa
        var _rsaBox = await Hive.openBox(getMD5('rsaBox'), encryptionCipher: HiveAesCipher(_hiveKey));
        var _realHiveKey;
        if(_rsaBox.containsKey(_rsaKeyMD5)){
          String encryptRsaKey = _rsaBox.get(_rsaKeyMD5);
          _rsaPublicKeyStr = encryptRsaKey.split(rsaSplit)[0];
          _rsaPrivateKeyStr = encryptRsaKey.split(rsaSplit)[1];

          _rsaPublicKey = rsaPublicKeyFromString(_rsaPublicKeyStr);
          _rsaPrivateKey = rsaPrivateKeyFromString(_rsaPrivateKeyStr);

          _rSAKeyEncrypt = Encrypter(RSA(publicKey: _rsaPublicKey, privateKey: _rsaPrivateKey));
          _realHiveKey = base64.decode(_rSAKeyEncrypt.decrypt64(_rsaBox.get(getMD5('realHiveKey'))));
        }else{
          RSAKeypair _rsaKeypair = RSAKeypair.fromRandom();
          _realHiveKey = Hive.generateSecureKey();
          _rsaPublicKey = _rsaKeypair.publicKey;
          _rsaPrivateKey = _rsaKeypair.privateKey;

          _rSAKeyEncrypt = Encrypter(RSA(publicKey: _rsaPublicKey, privateKey: _rsaPrivateKey));
          _rsaPublicKeyStr = rsaPublicKeyToString(_rsaPublicKey);
          _rsaPrivateKeyStr = rsaPrivateKeyToString(_rsaPrivateKey);

          await _rsaBox.put(_rsaKeyMD5, _rsaPublicKeyStr + rsaSplit + _rsaPrivateKeyStr);
          await _rsaBox.put(getMD5('realHiveKey'), _rSAKeyEncrypt.encrypt(base64.encode(_realHiveKey)).base64);
        }

        ///真正的加密box
        _realEncryptBox = await Hive.openBox(getMD5('rsaBox'), encryptionCipher: HiveAesCipher(_realHiveKey));
      }else if(_storageType == StorageType.SP){
        String _spKey;
        if(_prefs.containsKey(_rsaKeyMD5)){
          String encryptRsaKey = _prefs.get(_rsaKeyMD5);
          _rsaPublicKeyStr = encryptRsaKey.split(rsaSplit)[0];
          _rsaPrivateKeyStr = encryptRsaKey.split(rsaSplit)[1];

          _rsaPublicKey = rsaPublicKeyFromString(_rsaPublicKeyStr);
          _rsaPrivateKey = rsaPrivateKeyFromString(_rsaPrivateKeyStr);

          _rSAKeyEncrypt = Encrypter(RSA(publicKey: _rsaPublicKey, privateKey: _rsaPrivateKey));
        }else{
          RSAKeypair _rsaKeypair = RSAKeypair.fromRandom();
          _rsaPublicKey = _rsaKeypair.publicKey;
          _rsaPrivateKey = _rsaKeypair.privateKey;
          _rSAKeyEncrypt = Encrypter(RSA(publicKey: _rsaPublicKey, privateKey: _rsaPrivateKey));
          _rsaPublicKeyStr = rsaPublicKeyToString(_rsaPublicKey);
          _rsaPrivateKeyStr = rsaPrivateKeyToString(_rsaPrivateKey);
          _prefs.setString(_rsaKeyMD5, _rsaPublicKeyStr + rsaSplit + _rsaPrivateKeyStr);
        }

        _firstKeyMD5 = getMD5('spKey');
        if(_prefs.containsKey(_firstKeyMD5)){
          String encryptBase64 = _prefs.getString(_firstKeyMD5);
          _spKey = salsaEncrypt.decrypt64(_rSAKeyEncrypt.decrypt64(encryptBase64), iv: iv);
        }else{
          _spKey = Key.fromSecureRandom(32).base64;
          String hiveValueBase64 = _rSAKeyEncrypt.encrypt(salsaEncrypt.encrypt(_spKey, iv: iv).base64).base64;
          _prefs.setString(_firstKeyMD5, hiveValueBase64);
        }

        ///spKey才是真正的aes密码
        _realEncrypt = Encrypter(AES(Key.fromBase64(_spKey)));
      }
    }

  }


  /// ********************************************统一处理*************************************************************///
  // ignore: missing_return
  Future<void> _put(String key, String value){
    if(_storageType == StorageType.HIVE){
      return _realEncryptBox.put(key, value);
    }else if(_storageType == StorageType.SP){
      if(_encrypt){
        key = _realEncrypt.encrypt(key, iv: _aesIv).base64;
        value = _realEncrypt.encrypt(value, iv: _aesIv).base64;
      }
      return _prefs.setString(key, value);
    }
  }


  T _get<T>(String key, T value){
    T result = value;
    if(_storageType == StorageType.HIVE){
      result = _realEncryptBox.get(key, defaultValue: value);
    }else if(_storageType == StorageType.SP){
      if(_encrypt){
        key = _realEncrypt.encrypt(key, iv: _aesIv).base64;
      }
      String encryptValue = _prefs.getString(key);
      if(isEmpty(encryptValue)){
        result = value;
      }else{
        if(_encrypt){
          result = _realEncrypt.decrypt64(encryptValue, iv: _aesIv) as T;
        }else{
          result = encryptValue as T;
        }
      }
    }
    return result;
  }




  /// ********************************************************put************************************************************************************///

  Future<void> putString(String key, String value) {
    return _put(key, value);
  }

  Future<void> putBool(String key, bool value) {
    return _put(key, value.toString());
  }


  Future<void> putInt(String key, int value) {
    return _put(key, value.toString());
  }


  Future<void> putDouble(String key, double value) {
    return _put(key, value.toString());
  }



  Future<void> putObject(String key, dynamic object){
    return _put(key, jsonEncode(object));
  }


  Future<void> putList(String key, List list) {
    list = list.map((e) => json.encode(e)).toList();
    return _put(key, json.encode(list));
  }



  /// *********************************************************get***********************************************************************************///

  String getString(String key, {String defValue: ''}) {
    return _get(key, defValue);
  }


  bool getBool(String key, {bool defValue: false}) {
    return _get(key, defValue);
  }


  int getInt(String key, {int defValue: 0}) {
    return _get(key, defValue);
  }


  double getDouble(String key, {double defValue: 0.0}) {
    return _get(key, defValue);
  }



  T getObject<T>(String key, {T defValue}) {
    return _get(key, defValue);
  }


  List getList<T>(String key, {T f(Map v)}) {
    String encodeList = _get(key, json.encode([]));
    List<dynamic> _dynamicList = json.decode(encodeList);
    if(isEmpty(f)){
      return _dynamicList?.map((value) => value.toString())?.toList();
    }else{
      return _dynamicList?.map((value) {
        return f(value);
      })?.toList();
    }

  }


  /// ***********************************************************key信息*********************************************************************************///

  bool haveKey(String key) {
    bool haveKey = false;
    if(_storageType == StorageType.HIVE){
      haveKey = _realEncryptBox.containsKey(key);
    }else if(_storageType == StorageType.SP){
      if(_encrypt){
        haveKey = _prefs.containsKey(_realEncrypt.encrypt(key).base64);
      }else{
        haveKey = _prefs.containsKey(key);
      }
    }
    return haveKey;
  }


  // ignore: missing_return
  Set<String> getKeys() {
    if(_storageType == StorageType.HIVE){
      return _realEncryptBox.keys;
    }else if(_storageType == StorageType.SP){
      return _prefs.getKeys();
    }
  }



  /// ***********************************************************清除*********************************************************************************///
  /// remove.
  // ignore: missing_return
  Future<void> remove(String key) {
    if(_storageType == StorageType.HIVE){
      return _realEncryptBox.delete(key);
    }else if(_storageType == StorageType.SP){
      if(_encrypt){
        return _prefs.remove(_realEncrypt.encrypt(key).base64);
      }else{
        return _prefs.clear();
      }
    }
  }


  /// clear.
  // ignore: missing_return
  Future<void> clear() {
    if(_storageType == StorageType.HIVE){
      return _realEncryptBox.clear();
    }else if(_storageType == StorageType.SP){
      return _prefs.clear();
    }
  }

}
