import 'dart:convert';
import 'dart:io';
import 'dart:math';
import 'dart:typed_data';
import 'package:asn1lib/asn1lib.dart';
import 'package:crypto/crypto.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:hex/hex.dart';
import 'package:package_info/package_info.dart';
import 'package:path_provider/path_provider.dart';
import 'package:pointycastle/api.dart';
import 'package:pointycastle/asymmetric/api.dart';
import 'package:pointycastle/key_generators/api.dart';
import 'package:pointycastle/key_generators/rsa_key_generator.dart';
import 'package:pointycastle/random/fortuna_random.dart';
import 'package:pointycastle/export.dart';



  ///snackBar
  showSnackBar(BuildContext context, text){
    if(text is String){
      Scaffold.of(context).showSnackBar(SnackBar(content: Text("$text")));
    }else if(text is Widget){
      Scaffold.of(context).showSnackBar(SnackBar(content: text));
    }
  }


  /********************************ObjectUtils******************************/
  /// Returns true if the string is null or 0-length.
  bool isEmptyString(String str) {
    return str == null || str.isEmpty;
  }

  /// Returns true if the list is null or 0-length.
  bool isEmptyList(List list) {
    return list == null || list.isEmpty;
  }

  /// Returns true if there is no key/value pair in the map.
  bool isEmptyMap(Map map) {
    return map == null || map.isEmpty;
  }

  /// Returns true  String or List or Map is empty.
  bool isEmpty(Object object) {
    if (object == null) return true;
    if (object is String && object.isEmpty) {
      return true;
    } else if (object is List && object.isEmpty) {
      return true;
    } else if (object is Map && object.isEmpty) {
      return true;
    }
    return false;
  }

  /// Returns true String or List or Map is not empty.
  bool isNotEmpty(Object object) {
    return !isEmpty(object);
  }

  /// Returns true Two List Is Equal.
  bool twoListIsEqual(List listA, List listB) {
    if (listA == listB) return true;
    if (listA == null || listB == null) return false;
    int length = listA.length;
    if (length != listB.length) return false;
    for (int i = 0; i < length; i++) {
      if (!listA.contains(listB[i])) {
        return false;
      }
    }
    return true;
  }

  /// get length.
  int getLength(Object value) {
    if (value == null) return 0;
    if (value is String) {
      return value.length;
    } else if (value is List) {
      return value.length;
    } else if (value is Map) {
      return value.length;
    } else {
      return 0;
    }
  }


  getImagePath(String name, {String format: 'png'}){
    return 'assets/images/$name.$format';
  }


  String randomStr({int length = 10}){
    String alphabet = 'qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM';
    String left = '';
    for (var i = 0; i < length; i++) {
      left = left + alphabet[Random().nextInt(alphabet.length)];
    }
    return left;
  }



  double getDoubleFromStr(dynamic str, {double defDouble = 0}) {
    return isNumeric(str) ? double.parse(str) : defDouble;
  }

  int getIntFromStr(dynamic str, {int defInt = 0}) {
    return isNumeric(str) ? int.parse(str) : defInt;
  }

  bool isNumeric(String str) {
    try {
      if (isNotEmpty(str)) {
        if (double.parse(str) == 0) {
          return false;
        } else {
          return true;
        }
      } else {
        return false;
      }
    } on FormatException {
      return false;
    }
  }


  String getMD5(String input){
    var content = new Utf8Encoder().convert(input);
    var digest = md5.convert(content);
    return HEX.encode(digest.bytes);
  }



  /********************************File处理******************************/
  ///获取缓存文件夹
  Future<Directory> getCacheDirectory({String path = '', Future<Directory> directoryPath}) async{
    Directory filepath = isEmpty(directoryPath)? Platform.isAndroid? await getApplicationSupportDirectory() : await getTemporaryDirectory() : await directoryPath;
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    Directory directory = Directory(filepath.path + "/" + packageInfo.packageName + (isEmptyString(path)? '' : '/$path'));
    bool exists = await directory.exists();
    if (!exists) {
      await directory.create();
    }
    return directory;
  }



  ///清理文件夹
  void clearDirectory({@required Directory directory, bool isDeleteDirectory = false}){
    if (directory.existsSync()) {
      List<FileSystemEntity> files = directory.listSync();

      if (files.length > 0) {
        files.forEach((file) {
          ///当包含文件夹的时候
          if (file.statSync().type == FileSystemEntityType.directory)
            clearDirectory(directory: Directory(file.path), isDeleteDirectory: true);
          else
            file.deleteSync();
        });
      }
      if (isDeleteDirectory)
        directory.deleteSync();
    }
  }

  /********************************JSON处理******************************/


  /// Converts JSON string or JSON map [source] to object.
  T getObject<T>(Object source, T f(Map v)) {
    if (source == null || source.toString().isEmpty) return null;
    try {
      Map map;
      if (source is String) {
        map = json.decode(source);
      } else {
        map = source;
      }
      return map == null ? null : f(map);
    } catch (e) {
      print('JsonUtil convert error, Exception：${e.toString()}');
    }
    return null;
  }


  /// Converts JSON string or JSON map list [source] to object list.
  List<T> getObjectList<T>(Object source, T f(Map v)) {
    if (source == null || source.toString().isEmpty) return null;
    try {
      List list;
      if (source is String) {
        list = json.decode(source);
      } else {
        list = source;
      }
      return list?.map((value) {
        return f(value);
      })?.toList();
    } catch (e) {
      print('JsonUtil convert error, Exception：${e.toString()}');
    }
    return null;
  }



  bool get isInDebugMode {
    bool inDebugMode = false;
    assert(inDebugMode = true);//如果debug模式下会触发赋值
    return inDebugMode;
  }



  ///----------------------------------------------RSA公钥私钥----------------------------------------------------

  AsymmetricKeyPair<RSAPublicKey, RSAPrivateKey> generateRSAKeyPair({int bitLength = 2048}) {
    // Create an RSA key generator and initialize it
    final keyGen = RSAKeyGenerator()
      ..init(ParametersWithRandom(RSAKeyGeneratorParameters(BigInt.parse('65537'), bitLength, 64), _secureRandom));
    // Use the generator
    final pair = keyGen.generateKeyPair();
    // Cast the generated key pair into the RSA key types
    final myPublic = pair.publicKey as RSAPublicKey;
    final myPrivate = pair.privateKey as RSAPrivateKey;
    return AsymmetricKeyPair<RSAPublicKey, RSAPrivateKey>(myPublic, myPrivate);
  }


  SecureRandom get _secureRandom{
    final secureRandom = FortunaRandom();
    final seedSource = Random.secure();
    final seeds = <int>[];
    for (int i = 0; i < 32; i++) {
      seeds.add(seedSource.nextInt(255));
    }
    secureRandom.seed(KeyParameter(Uint8List.fromList(seeds)));
    return secureRandom;
  }


  /// Create an [RSAPublicKey] from the given String.
  RSAPublicKey rsaPublicKeyFromString(String publicKeyString) {
    List<int> publicKeyDER = base64Decode(publicKeyString);
    final asn1Parser = ASN1Parser(publicKeyDER as Uint8List);
    final topLevelSeq = asn1Parser.nextObject() as ASN1Sequence;
    final publicKeyBitString = topLevelSeq.elements[1];

    final publicKeyAsn = ASN1Parser(publicKeyBitString.contentBytes());
    final publicKeySeq = publicKeyAsn.nextObject() as ASN1Sequence;
    final modulus = publicKeySeq.elements[0] as ASN1Integer;
    final exponent = publicKeySeq.elements[1] as ASN1Integer;

    return RSAPublicKey(modulus.valueAsBigInteger, exponent.valueAsBigInteger);
  }


  RSAPublicKey rsaPublicKeyFromPEM(String pemString) {
    final rows = pemString.split(RegExp(r'\r\n?|\n'));
    final privateKeyString = rows
        .skipWhile((row) => row.startsWith('-----BEGIN'))
        .takeWhile((row) => !row.startsWith('-----END'))
        .map((row) => row.trim())
        .join('');
    return rsaPublicKeyFromString(privateKeyString);
  }



  String rsaPublicKeyToString(RSAPublicKey _publicKey) {
    final algorithmSeq = ASN1Sequence();
    final algorithmAsn1Obj = ASN1Object.fromBytes(Uint8List.fromList(
        [0x6, 0x9, 0x2a, 0x86, 0x48, 0x86, 0xf7, 0xd, 0x1, 0x1, 0x1]));
    final paramsAsn1Obj = ASN1Object.fromBytes(Uint8List.fromList([0x5, 0x0]));
    algorithmSeq.add(algorithmAsn1Obj);
    algorithmSeq.add(paramsAsn1Obj);

    final publicKeySeq = ASN1Sequence();
    publicKeySeq.add(ASN1Integer(_publicKey.modulus));
    publicKeySeq.add(ASN1Integer(_publicKey.exponent));
    final publicKeySeqBitString =
    ASN1BitString(Uint8List.fromList(publicKeySeq.encodedBytes));

    final topLevelSeq = ASN1Sequence();
    topLevelSeq.add(algorithmSeq);
    topLevelSeq.add(publicKeySeqBitString);
    return base64.encode(topLevelSeq.encodedBytes);
  }

  /// Export a [RSAPublicKey] as PEM String which can be reversed using [RSAPublicKey.fromPEM].
  String rsaPublicKeyToPEM(RSAPublicKey publicKey) {
    return '-----BEGIN PUBLIC KEY-----\n${rsaPublicKeyToString(publicKey)}\n-----END PUBLIC KEY-----';
  }

  /// Export a [RSAPublicKey] as formatted PEM String which can be reversed using [RSAPublicKey.fromPEM].
  String rsaPublicKeyToFormattedPEM(RSAPublicKey publicKey) {
    final base = rsaPublicKeyToString(publicKey);
    var formatted = '';
    for (var i = 0; i < base.length; i++) {
      if (i % 64 == 0 && i != 0) {
        formatted += '\n';
      }
      formatted += base[i];
    }
    return '-----BEGIN PUBLIC KEY-----\n$formatted\n-----END PUBLIC KEY-----';
  }



  /// Create an [RSAPrivateKey] from the given String.
  RSAPrivateKey rsaPrivateKeyFromString(String privateKeyString) {
    List<int> privateKeyDER = base64Decode(privateKeyString);
    var asn1Parser = ASN1Parser(privateKeyDER as Uint8List);
    final topLevelSeq = asn1Parser.nextObject() as ASN1Sequence;
    final privateKey = topLevelSeq.elements[2];

    asn1Parser = ASN1Parser(privateKey.contentBytes());
    final pkSeq = asn1Parser.nextObject() as ASN1Sequence;

    final modulus = pkSeq.elements[1] as ASN1Integer;
    final privateExponent = pkSeq.elements[3] as ASN1Integer;
    final p = pkSeq.elements[4] as ASN1Integer;
    final q = pkSeq.elements[5] as ASN1Integer;

    return RSAPrivateKey(modulus.valueAsBigInteger, privateExponent.valueAsBigInteger, p.valueAsBigInteger, q.valueAsBigInteger);
  }

  /// Create an [RSAPrivateKey] from the given PEM-String.
  RSAPrivateKey rsaPrivateKeyFromPEM(String pemString) {
    final rows = pemString.split(RegExp(r'\r\n?|\n'));
    final privateKeyString = rows
        .skipWhile((row) => row.startsWith('-----BEGIN'))
        .takeWhile((row) => !row.startsWith('-----END'))
        .map((row) => row.trim())
        .join('');
    return rsaPrivateKeyFromString(privateKeyString);
  }




  String rsaPrivateKeyToString(RSAPrivateKey privateKey) {
    final version = ASN1Integer(BigInt.from(0));

    final algorithmSeq = ASN1Sequence();
    final algorithmAsn1Obj = ASN1Object.fromBytes(Uint8List.fromList(
        [0x6, 0x9, 0x2a, 0x86, 0x48, 0x86, 0xf7, 0xd, 0x1, 0x1, 0x1]));
    final paramsAsn1Obj = ASN1Object.fromBytes(Uint8List.fromList([0x5, 0x0]));
    algorithmSeq.add(algorithmAsn1Obj);
    algorithmSeq.add(paramsAsn1Obj);

    final privateKeySeq = ASN1Sequence();
    final modulus = ASN1Integer(privateKey.n);
    final publicExponent = ASN1Integer(BigInt.parse('65537'));
    final privateExponent = ASN1Integer(privateKey.privateExponent);
    final p = ASN1Integer(privateKey.p);
    final q = ASN1Integer(privateKey.q);
    final dP = privateKey.privateExponent % (privateKey.p - BigInt.from(1));
    final exp1 = ASN1Integer(dP);
    final dQ = privateKey.privateExponent % (privateKey.q - BigInt.from(1));
    final exp2 = ASN1Integer(dQ);
    final iQ = privateKey.q.modInverse(privateKey.p);
    final co = ASN1Integer(iQ);

    privateKeySeq.add(version);
    privateKeySeq.add(modulus);
    privateKeySeq.add(publicExponent);
    privateKeySeq.add(privateExponent);
    privateKeySeq.add(p);
    privateKeySeq.add(q);
    privateKeySeq.add(exp1);
    privateKeySeq.add(exp2);
    privateKeySeq.add(co);
    final publicKeySeqOctetString =
    ASN1OctetString(Uint8List.fromList(privateKeySeq.encodedBytes));

    final topLevelSeq = ASN1Sequence();
    topLevelSeq.add(version);
    topLevelSeq.add(algorithmSeq);
    topLevelSeq.add(publicKeySeqOctetString);
    return base64.encode(topLevelSeq.encodedBytes);
  }

  /// Export a [RSAPrivateKey] as PEM String which can be reversed using [RSAPrivateKey.fromPEM].
  String rsaPrivateKeyToPEM(RSAPrivateKey privateKey) {
    return '-----BEGIN RSA PRIVATE KEY-----\n${rsaPrivateKeyToString(privateKey)}\n-----END RSA PRIVATE KEY-----';
  }

  /// Export a [RSAPrivateKey] as formatted PEM String which can be reversed using [RSAPrivateKey.fromPEM].
  String rsaPrivateKeyToFormattedPEM(RSAPrivateKey privateKey) {
    final base = rsaPrivateKeyToString(privateKey);
    var formatted = '';
    for (var i = 0; i < base.length; i++) {
      if (i % 64 == 0 && i != 0) {
        formatted += '\n';
      }
      formatted += base[i];
    }
    return '-----BEGIN RSA PRIVATE KEY-----\n$formatted\n-----END RSA PRIVATE KEY-----';
  }

