import 'package:extended_image/extended_image.dart';
import 'package:extended_list/extended_list.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_mylibrary/base/base_view_state.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/utils/win_media.dart';
import 'package:flutter_mylibrary/widget/textformfield_widget.dart';


Widget buildNoDataWidget({BaseViewState viewStateModel, Widget child, GestureDetector onTap}) => Container(
  height: double.infinity,
  alignment: Alignment.center,
  margin: getEdgeInsetsSymmetric(horizontal: defMargin),
  child: child??GestureDetector(
    onTap: onTap??(){
      viewStateModel.requestWhenError();
    },
    child: Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        Spacer(),
        SizedBox(
            width: getWidth(100),
            height: getHeight(100),
            child: Image.asset('assets/images/nodata.png')
        ),
        Text(
          viewStateModel.httpError.message??UNKNOWN_MSG,
          style: TextStyle(fontSize: getSp(16), color: Colors.grey[400]),
        ),
        Spacer(),
      ],
    ),
  ),
);


Widget buildViewStateWidget(BaseViewState viewState, Function idleWidget, {Widget busyWidget, Widget emptyDataWidget, Widget errorDataWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
  if(viewState.isIdle){
    return idleWidget();
  }else if(viewState.isError){
    if (showErrorWidget) {
      return errorDataWidget??errorWidget(viewState);
    } else{
      return idleWidget();
    }
  }else if (viewState.isBusy) {
    if (showBusyWidget) {
      return busyWidget??pageLoading;
    }else{
      return idleWidget();
    }
  } else{
    return Container();
  }
}

Widget buildRefreshNoDataWidget({BaseViewState viewState, Widget busyWidget, Widget emptyDataWidget, Widget errorDataWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
  if(viewState.isError && showErrorWidget){
    return errorDataWidget??errorWidget(viewState);
  }else if (viewState.isBusy && showBusyWidget) {
    return busyWidget??pageLoading;
  }else{
    return null;
  }
}





/// 间隔
class Gaps {
  /// 水平间隔
  static Widget hGap5 = new SizedBox(width: getWidth(5));
  static Widget hGap10 = new SizedBox(width: getWidth(10));
  static Widget hGap15 = new SizedBox(width: getWidth(15));
  static Widget hGapDef = new SizedBox(width: getWidth(defMargin));

  /// 垂直间隔
  static Widget vGap5 = new SizedBox(height: getHeight(5));
  static Widget vGap10 = new SizedBox(height: getHeight(10));
  static Widget vGap15 = new SizedBox(height: getHeight(15));
  static Widget vGapDef = new SizedBox(width: getHeight(defMargin));


  /// 自定义垂直间隔
  static Widget getVertical(double height){
    return SizedBox(
      height: getHeight(height),
    );
  }

  /// 自定义水平间隔
  static Widget getHorizontal(double width){
    return SizedBox(
      width: getHeight(width),
    );
  }
}



/// ******************************公共控件******************************/
Widget buildAppBarWidget({Color backgroundColor, Widget leading, bool automaticallyImplyLeading, TextStyle textStyle, Widget titleWidget, double titleTextSize = 16, FontWeight fontWeight, bool centerTitle = true, String title = '', Widget rightWidget, GestureTapCallback onRightWidgetTap, double titleSpacing, PreferredSizeWidget bottom, double leadingWidth, double toolbarHeight}){
  double horizontalMargin = getWidth(titleSpacing??defMargin);
  return AppBar(
    toolbarHeight: toolbarHeight,
    leadingWidth: leadingWidth,
    automaticallyImplyLeading: automaticallyImplyLeading??(!isEmptyString(title) && centerTitle),
    titleSpacing: horizontalMargin,
    title: titleWidget??buildText(title, fontSize: getSp(titleTextSize), fontWeight: fontWeight, textStyle: textStyle??Get.theme.appBarTheme.titleTextStyle),
    bottom: bottom,
    leading: leading,
    centerTitle: centerTitle,
    backgroundColor: backgroundColor??Get.theme.appBarTheme.backgroundColor,
    actions: [
      GestureDetector(
        onTap: onRightWidgetTap,
        child: Align(
          alignment: Alignment.center,
          child: Padding(
            padding: getEdgeInsetsOnly(right: horizontalMargin),
            child: rightWidget,
          ),
        ),
      )
    ],
  );
}



///获取SafeArea的布局
Widget buildScaffoldWidget({Color backgroundColor, Color appBarBackColor, Widget leading, bool useAppBar = true,bool automaticallyImplyLeading, TextStyle textStyle, Widget titleWidget, double titleTextSize = 16, FontWeight fontWeight, bool centerTitle = true, @required Widget body, String title = '', Widget rightWidget, GestureTapCallback onRightWidgetTap, double titleSpacing, EdgeInsets padding, Widget bottomNavigationBar, double leadingWidth, PreferredSizeWidget bottom, double toolbarHeight = kToolbarHeight, bool useSafeArea = true, bool resizeToAvoidBottomInset = false}){
  return Scaffold(
    backgroundColor: backgroundColor??Get.theme.canvasColor,
    appBar: useAppBar? buildAppBarWidget(
      toolbarHeight: toolbarHeight,
      leadingWidth: leadingWidth,
      title: title,
      automaticallyImplyLeading: automaticallyImplyLeading,
      titleWidget: titleWidget,
      fontWeight: fontWeight,
      leading: leading,
      titleTextSize: titleTextSize,
      rightWidget: rightWidget,
      onRightWidgetTap: onRightWidgetTap,
      titleSpacing: titleSpacing,
      centerTitle: centerTitle,
      textStyle: textStyle,
      backgroundColor: appBarBackColor,
      bottom: bottom
    ) : null,
    resizeToAvoidBottomInset: resizeToAvoidBottomInset,
    bottomNavigationBar: bottomNavigationBar,
    body: useSafeArea? SafeArea(
      child: Padding(
        padding: padding??getEdgeInsetsAll(0),
        child: body,
      ),
    ) : Padding(
      padding: padding??getEdgeInsetsAll(0),
      child: body,
    ),
  );
}



Widget buildText(String text, {TextStyle textStyle, int maxLines,double fontSize = 14, Color color, EdgeInsets margin, TextDecoration textDecoration, TextAlign textAlign, FontWeight fontWeight, TextOverflow overflow}){
  return Container(
    margin: margin,
    child: Text(
      text??'',
      maxLines: maxLines,
      overflow: overflow,
      textAlign: textAlign,
      style: textStyle??TextStyle(
          decoration: textDecoration,
          fontSize: getSp(fontSize),
          fontWeight: fontWeight,
          color: color??Theme.of(Get.context).textTheme.bodyText1.color
      ),
    ),
  );
}





Widget buildTextField({@required TextEditingController controller, Color cursorColor, double hintFontSize = 12, TextStyle hintStyle, bool autofocus = false, InputDecoration decoration, String hintText, int maxLength, EdgeInsets margin, TextAlign textAlign = TextAlign.left, bool enabled = true, double fontSize = 14, TextInputType textInputType, int maxLines, List<TextInputFormatter> textInputFormatters, FocusNode focusNode, bool obscureText, TextStyle textStyle}){
  return Container(
    margin: margin,
    child: TextField(
      textInputAction: TextInputAction.done,
      focusNode: focusNode,
      maxLines: isEmpty(obscureText)? maxLines : isEmpty(maxLines)? 1 : maxLines,
      enabled: enabled,
      textAlign: textAlign,
      autofocus: autofocus,
      style: textStyle??TextStyle(fontSize: getSp(fontSize), color: Colors.black),
      maxLength: maxLength,
      keyboardType: textInputType,
      obscureText: obscureText??false,
      inputFormatters: textInputFormatters,
      cursorColor: cursorColor??Colors.blue,
      decoration: decoration??InputDecoration.collapsed(
        border: InputBorder.none,
        hintText: hintText,
        hintStyle: hintStyle??TextStyle(
          fontSize: getSp(hintFontSize),
          color: Colors.grey
        )
      ),
      cursorRadius: Radius.circular(3),
      controller: controller,
    ),
  );
}


Widget buildTextWithClearField(TextEditingController textEditingController, double suffixIconSize, {FocusNode focusNode, int maxLines, String hintText, int maxLength, EdgeInsets margin, TextAlign textAlign = TextAlign.left, bool enabled = true, double fontSize = 14, IconData clearIcon, Widget icon}){
  return buildTextField(
    margin: margin,
    enabled: enabled,
    textAlign: textAlign,
    fontSize: fontSize,
    maxLength: maxLength,
    maxLines: maxLines,
    focusNode: focusNode,
    decoration: InputDecoration(
      icon: icon,
      border: InputBorder.none,
      hintText: hintText,
      suffixIcon: TextFieldClearWidget(textEditingController, suffixIconSize, iconData: clearIcon,),
    ),
    controller: textEditingController,
  );
}




///间隔listview
Widget buildListViewSeparated({bool closeToTrailing = false, ViewPortBuilder viewportBuilder, @required int itemCount, @required IndexedWidgetBuilder itemBuilder, Axis scrollDirection = Axis.vertical, double divierHeight = 15, Color divierColor, ScrollPhysics physics, EdgeInsets padding,  bool useSliver = false, bool shrinkWrap = false, bool useListView = true, CollectGarbage collectGarbage}){

  if (useSliver) {
    if (itemCount <= 0) {
      return SliverToBoxAdapter(
        child: Container(
          width: 0,
          height: 0,
        ),
      );
    }
    if (useListView) {
      if (scrollDirection == Axis.vertical) {
        return ExtendedSliverList(
          delegate: SliverChildBuilderDelegate(
                  (context, index){
                if(index.isOdd){
                  return Container(
                    height: getHeight(divierHeight),
                    color: divierColor,
                  );
                }
                return Padding(
                  padding: padding??getEdgeInsetsAll(0),
                  child: itemBuilder(context, index ~/ 2),
                );
              },
              childCount: (itemCount * 2 ) - 1
          ),
          extendedListDelegate: ExtendedListDelegate(
            ///追踪进入 Viewport 的列表元素的 index（即你看到的可视区域，并不包括缓存距离）
              viewportBuilder: (int fistIndex, int lastIndex){
                if(isNotEmpty(viewportBuilder)){
                  viewportBuilder(fistIndex, lastIndex);
                }
              },
              closeToTrailing: closeToTrailing,
              collectGarbage: collectGarbage
          ),
        );
      }else{
        return SliverToBoxAdapter(
          child: ExtendedListView.separated(
              padding: padding??getEdgeInsetsAll(0),
              shrinkWrap: shrinkWrap,
              physics: physics,
              itemCount: itemCount,
              scrollDirection: scrollDirection,
              separatorBuilder: (BuildContext context, int index){
                return Container(
                  height: getHeight(divierHeight),
                  color: divierColor,
                );
              },
              extendedListDelegate: ExtendedListDelegate(
                ///追踪进入 Viewport 的列表元素的 index（即你看到的可视区域，并不包括缓存距离）
                  viewportBuilder: (int fistIndex, int lastIndex){
                    if(isNotEmpty(viewportBuilder)){
                      viewportBuilder(fistIndex, lastIndex);
                    }
                  },
                  closeToTrailing: closeToTrailing,
                  collectGarbage: collectGarbage
              ),

              itemBuilder: itemBuilder
          ),
        );
      }
    } else{
      List<Widget> widgetList = List.generate((itemCount * 2) - 1, (index){
        if(index.isOdd){
          return Container(
            height: getHeight(divierHeight),
            color: divierColor,
          );
        }
        return Padding(
          padding: padding??getEdgeInsetsAll(0),
          child: itemBuilder(Get.context, index ~/ 2),
        );
      });

      ///使用Column或者Row
      return SliverToBoxAdapter(
        child: SingleChildScrollView(
          scrollDirection: scrollDirection,
          physics: physics,
          child: scrollDirection == Axis.vertical? Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList,
          ) : Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList,
          ),
        ),
      );
    }
  }else{
    if (itemCount <= 0) {
      return Container(
        width: 0,
        height: 0,
      );
    }
    if (useListView) {
      return ExtendedListView.separated(
          padding: padding??getEdgeInsetsAll(0),
          shrinkWrap: shrinkWrap,
          physics: physics,
          itemCount: itemCount,
          scrollDirection: scrollDirection,
          separatorBuilder: (BuildContext context, int index){
            return Container(
              height: getHeight(divierHeight),
              color: divierColor,
            );
          },
          extendedListDelegate: ExtendedListDelegate(
            ///追踪进入 Viewport 的列表元素的 index（即你看到的可视区域，并不包括缓存距离）
              viewportBuilder: (int fistIndex, int lastIndex){
                if(isNotEmpty(viewportBuilder)){
                  viewportBuilder(fistIndex, lastIndex);
                }
              },
              closeToTrailing: closeToTrailing,
              collectGarbage: collectGarbage
          ),
          itemBuilder: itemBuilder
      );
    }else{
      List<Widget> widgetList = List.generate((itemCount * 2 ) - 1, (index){
        if(index.isOdd){
          return Container(
            width: scrollDirection == Axis.horizontal ? getWidth(divierHeight) : null,
            height: scrollDirection == Axis.vertical ? getHeight(divierHeight) : null,
            color: divierColor,
          );
        }
        return Padding(
          padding: padding??getEdgeInsetsAll(0),
          child: itemBuilder(Get.context, index ~/ 2),
        );
      });
      ///使用Column或者Row
      return SingleChildScrollView(
        scrollDirection: scrollDirection,
        physics: physics,
        child: scrollDirection == Axis.vertical? Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList
        ) : Row(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: widgetList
        ),
      );
    }
  }

}





///胶囊控件
Widget buildConnerContainer({bool userClip = false, AlignmentGeometry alignment, GestureTapCallback onTap, bool isCircle = false, double borderWidth = 0, Color backgroundColor = Colors.white, double radius = 0, Widget child, EdgeInsets padding, Color borderColor = Colors.white, EdgeInsets margin, BorderRadiusGeometry borderRadius, double height, double width, HitTestBehavior behavior}){
  return GestureDetector(
      behavior: behavior,
      onTap: onTap,
      child: userClip? ClipRRect(
        borderRadius: borderRadius??BorderRadius.all(Radius.circular(radius)),
        child: Container(
          color: backgroundColor,
          width: width,
          height: height,
          margin: margin,
          padding: padding,
          alignment: alignment,
          child: child??Container(),
        ),
      ): Container(
        width: width,
        height: height,
        margin: margin,
        padding: padding,
        alignment: alignment,
        decoration: BoxDecoration(
            shape: isCircle? BoxShape.circle : BoxShape.rectangle,
            color: backgroundColor,
            borderRadius: isCircle? null : borderRadius??BorderRadius.all(Radius.circular(radius)),
            border: borderWidth == 0? null : Border.all(color: borderColor, width: borderWidth)
        ),
        child: child??Container(),
      )
  );
}




///阴影框
Widget buildBoxShadow({GestureTapCallback onTap, AlignmentGeometry alignment, double height, double width, Offset offset, Color boxColor, double blurRadius = 10, BorderRadiusGeometry borderRadiusGeometry, Widget child, EdgeInsetsGeometry padding, BoxBorder border, EdgeInsets margin, double radius = 5}){
  return GestureDetector(
    onTap: onTap,
    child: Container(
        margin: margin,
        padding: padding,
        width: width,
        height: height,
        alignment: alignment,
        decoration: BoxDecoration(
            border: border,
            color: Theme.of(Get.context).canvasColor,
            boxShadow: [
              BoxShadow(
                offset: offset??Offset(5.0, 5.0),
                color: boxColor??Color(0xff9cb3b3b3),//阴影颜色
                blurRadius: blurRadius,//阴影大小
              )
            ],
            borderRadius: borderRadiusGeometry??BorderRadius.all(Radius.circular(radius))
        ),
        child: child
    ),
  );
}





///圆角图片
Widget buildConnerNetworkImage({LoadStateChanged loadStateChanged, bool enableLoadState = false, bool cache = true, bool enableMemoryCache = true, bool clearMemoryCacheIfFailed = true,String url, double width, double height, bool isCircle = false, double radius = 5, BoxFit fit = BoxFit.fill}){
  return ExtendedImage(
    image: ExtendedNetworkImageProvider(
      url,
      cache: cache,
    ),
    width: isEmpty(width)? null : getWidth(width),
    height: isEmpty(height)? null : getHeight(height),
    shape: isCircle? BoxShape.circle : BoxShape.rectangle,
    borderRadius: BorderRadius.all(Radius.circular(radius)),
    enableMemoryCache: enableMemoryCache,
    clearMemoryCacheIfFailed: clearMemoryCacheIfFailed,
    enableLoadState: enableLoadState,
    loadStateChanged: loadStateChanged,
    fit: fit,
  );
}

///获取以左侧为高度，右侧控件为上下对齐的布局
Widget buildFlexRowWidget({Widget leftWidget, Widget rightWidgetTopWidget, Widget rightBottomWidget, double divider}){
  return IntrinsicHeight(
    child: Row(
      children: [
        leftWidget,
        Gaps.getHorizontal(divider),
        Expanded(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Expanded(
                child: rightWidgetTopWidget,
              ),
              rightBottomWidget
            ],
          ),
        )
      ],
    ),
  );
}



